@@ -11,6 +11,7 @@ mod use_drop;
 mod use_effect_once;
 mod use_effect_update;
 mod use_event;
+mod use_favicon;
 mod use_geolocation;
 mod use_hash;
 mod use_interval;
@@ -39,6 +40,7 @@ mod use_size;
 mod use_state_ptr_eq;
 mod use_swipe;
 mod use_throttle;
+mod use_throttle_effect;
 mod use_throttle_state;
 mod use_timeout;
 mod use_title;
@@ -62,6 +64,7 @@ pub use use_drop::*;
 pub use use_effect_once::*;
 pub use use_effect_update::*;
 pub use use_event::*;
+pub use use_favicon::*;
 pub use use_geolocation::*;
 pub use use_hash::*;
 pub use use_interval::*;
@@ -90,6 +93,7 @@ pub use use_size::*;
 pub use use_state_ptr_eq::*;
 pub use use_swipe::*;
 pub use use_throttle::*;
+pub use use_throttle_effect::*;
 pub use use_throttle_state::*;
 pub use use_timeout::*;
 pub use use_title::*;
